// A simple program to generate exponential random numbers and
// store them in a histogram; also optionally writes the individual
// values to a file.

// Glen Cowan
// RHUL Physics
// 2 December 2006

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <string>
#include <cmath>

#include <TFile.h>
#include <TH1D.h>
#include <TRandom3.h>

using namespace std;

int main(int argc, char **argv) {

// Set up output files, book histograms, add to list of histograms.

  TFile* histFile = new TFile("expData.root", "RECREATE");
  TList* hList = new TList();      // list of histograms to store
  TH1D* h1 = new TH1D("h1", "exponential", 100, 0.0, 10.0);
  hList->Add(h1);

  string answer;
  ofstream dataFile;
  cout << "Also store individual values in a file? (y/n)" << endl;
  cin >> answer;
  bool makeDataFile = (answer == "y" || answer == "Y");
  if ( makeDataFile ) { dataFile.open("expData.txt"); } 

// Create a TRandom object to generate random numbers uniform in ]0,1]
// Use the "Marsenne Twister" algorithm of TRandom3

  int seed = 12345;
  TRandom* ran = new TRandom3(seed);
 
//  Fill with exponential random numbers.

  const double xi = 1.0;              // mean value of the exponential
  int numVal = 0;
  cout << "Enter number of values to generate: ";
  cin >> numVal;

  for (int i = 0; i<numVal; ++i){
    double r = ran->Rndm();
    double x = - xi * log(r);
    h1->Fill(x);
    if ( makeDataFile ) { dataFile << x << endl; }
  }

// Save all histograms and close up.

  hList->Write();
  histFile->Close();
  if ( makeDataFile ) { dataFile.close(); }

  return 0;

}
